
// Foldees.java
// Andrew Davison, Feb 2017, ad@fivedots.coe.psu.ac.th

/* DC Comic Book Foldees in action!
   Published by Topps in 1966

   This series of 44 cards folds out to a size of 2-1/2" x 4-11/16": 
   "Make 9 Funny Pictures with Every Card". 

   A larger (folding out to 3-1/4" x 6") set was produced later, 
   without card #44 or the wording "Printed in U.S.A." after the 
   Topps copyright mark. 

   American Card Catalog reference is R708-12. 
   Scans are posted at the Vintage Non-Sports Cards gallery
   and http://www1.coe.neu.edu/~dan/fold-66/go.html
*/


import java.awt.*;
import javax.swing.*;
import java.util.*;
import java.awt.event.*;
import java.awt.image.*;
import java.io.*;
import javax.imageio.*;


public class Foldees extends JFrame
{
  private static final int CARD_WIDTH = 720;
  private static final int CARD_HEIGHT = 380;

  // card regions
  private static final int FRONT = 0;
  private static final int BACK = 1;

  private static final int LEFT = 0;
  private static final int CENTER = 1;
  private static final int RIGHT = 2;

  private static final int TOP = 0;
  private static final int BOTTOM = 1;

  // many thanks to Mike Kooiman for this list
  private static final String names[] = {
       " 1. Speedy Flash / Western Explorer / Evil Devil",
       " 2. Jimmy Olsen Cub Reporter / Graceful Dancer / Grubby Beatnik",
       " 3. Super Hero (Superman) / Fat Lady / Pistol-Packin' Marshall",
       " 4. Youthful Kid Flash / Viking Warrior / Flea-Bitten Dog",
       " 5. Heroic Green Arrow / Beautiful Starlet / Crying Baby",
       " 6. Riddler / Bear Named Sam Escapes from Park Zoo! / Slugger Hits 8 Balls out of Baseball Bark",
       " 7. Speedy Old Flash / Evil Witch Doctor / British Queen ",
       " 8. Powerful Green Lantern / Scary Frankenstein / Beautiful Bathing Beauty",
       " 9. Atom / Mahatma Gandhi / Fido the Pooch",
       "10. Handsome Green Lantern / Cross Eyed Clod / Beautiful Betsy Ross",
       "11. Super Krypto / Clucking Chicken / Dancing Elephant",
       "12. Wonder Woman the Amazon / Wanda the Witch / Babe Ruth the Slugger",
       "13. High-Flying Supergirl / Roman Emperor / Sloppy Hobo",
       "14. Mischievous Imp (Mxyzptlk) / Western Marshall / Snappy Hot Rodder",
       "15. Martian Man Hunter / Loud Singer / Indian Chief",
       "16. Saturn Girl The Legionnaire / Ulysses S. Grant The General / Pork Chops The Pig",
       "17. Crime-Fighting Batman / Honest President / Fanged Dracula",
       "18. Penguin the Evil Villain / Sinbad The Sailor / Lawrence of Arabia",
       "19. Gold, the Metal Man / Oriental Criminal / Leo, the Lion",
       "20. Alfred the Bulter / Blackbeard the Pirate / Hoppy the Kangaroo",
       "21. Evil Joker / Bucking Bronco / Pretty Skater",
       "22. High-Flying Hawkgirl / Dutch Governor / Western Cowboy",
       "23. Adventurous Supergirl / Silent Comedian / Russian Politician",
       "24. Batmobile / Bath Tub / Jack-in-the box",
       "25. Crusading Editor (Perry White) / Decaying Mummy / Fighting Soldier",
       "26. Superman / Jet Plane / Spotted Pup",
       "27. Sharp-Shooting Green Arrow / Mysterious Secret Agent / Barnyard Pig",
       "28. Ugly Bizarro / Russian Dancer / Shouting Paul Revere",
       "29. Evil Penguin / Charity Raffles / Scientist Finds Cure...",
       "30. Leader of the Blackhawks / Queen of the Egyptians / Mangy Rats",
       "31. Robin Boy Wonder / Benjamin Franklin / Metal Robot",
       "32. Winged Hawkman / Prehistoric Cave Man / Aging Scrubwoman",
       "33. Beautiful Wonder Woman / Rock n Roll Singer / Dopey Dunce",
       "34. Scheming Riddler / Hawaiian Dancer / Bizarre Martian",
       "35. Leering Luthor / Happy Singer /  Pretty Actress",
       "36. Stretching Hero (Elongated Man) / Kiddie Bicyclist / Smiling Pianist",
       "37. Metamorpho the Element Man / Balancing Bicyclist / Swimming Dolphin",
       "38. Elastic Plastic Man / Graceful Giraffe / Chief Sitting Bull",
       "39. Comic Book Reporter (Lois Lane) / Cuban Dictator / 18th Century President",
       "40. The Aquaman / Beautiful Starlet / Ugly Monster",
       "41. Flash / Ape Man in the Jungle / Astronaut in Space",
       "42. Spectre / 19th Century Champion / Bad Tempered Hermit",
       "43. Catwoman / DeGaule the General / Tipsy Drunk",
       "44. Superman / Baseball Hero Autographing Ball / Boy Eating Ice Cream Cone"
     };

  // GUI components
  private JComboBox<String> namesCB;
  private JButton cardJB, facesJB, refoldJB;
  private ImagePanel foldeePane, frontCard, backCard;

  private Random rand = new Random();  
  private int currTPos, currBPos, topPos, botPos;
            // x- positions used for top and bottom of card
  private BufferedImage frontIm, backIm;


  public Foldees()
  {
    super("Play with DC Comic Book Foldees");

    Container c = getContentPane();
    c.setLayout( new BoxLayout(c, BoxLayout.Y_AXIS));

    // ---------- two rows of controls ----------------
    JPanel p1 = new JPanel(new FlowLayout(FlowLayout.LEFT));
    p1.add( new JLabel("Choose a card: "));
    namesCB = new JComboBox<String>(names);
    namesCB.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e)
      { if(e.getStateChange() == ItemEvent.SELECTED)
          useCard( namesCB.getSelectedIndex()+1);
      }
    });
    p1.add(namesCB);
    c.add(p1);

    JPanel p2 = new JPanel(new FlowLayout(FlowLayout.LEFT));
    p2.add( new JLabel("Or randomly choose: "));

    cardJB = new JButton("A Card");
    cardJB.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) 
      {  useRandomCards(1);  }
    });

    facesJB = new JButton("Two Faces");
    facesJB.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) 
      {  useRandomCards(2);  }
    });

    p2.add(cardJB);
    p2.add(facesJB);
    c.add(p2);

    // ---- two column display ------------
    JPanel p3 = new JPanel();
    p3.setLayout( new BoxLayout(p3, BoxLayout.X_AXIS));


    // ----- left display ------
    JPanel p3Left = new JPanel();
    p3Left.setLayout( new BoxLayout(p3Left, BoxLayout.Y_AXIS));

    // front card 
    frontCard = new ImagePanel(CARD_WIDTH/2, CARD_HEIGHT/2);
    p3Left.add(frontCard);

    // back card 
    backCard = new ImagePanel(CARD_WIDTH/2, CARD_HEIGHT/2);
    p3Left.add(backCard);

    refoldJB = new JButton("Refold");
    refoldJB.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) 
      {  randomFold();  }
    });

    p3Left.add(refoldJB);

    p3.add(p3Left);

    // ----- right display ------

    foldeePane = new ImagePanel(CARD_WIDTH/3, CARD_HEIGHT);
    p3.add(foldeePane);

    c.add(p3);

    currTPos = -1; currBPos = -1;
    topPos = -1; botPos = -1;
    // initialize display with card no. 1
    useCard(1);

    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    pack();
    setResizable(false);
    setLocationRelativeTo(null);  // center the window
    setVisible(true);
  } // end of Foldees()


  // --------- process GUI input ----------------


  private void useCard(int cardIdx)
  {
    // System.out.println("useCard: " + cardIdx);
    frontIm = loadCardImage(cardIdx, FRONT);
    frontCard.setImage(frontIm);

    backIm = loadCardImage(cardIdx, BACK);
    backCard.setImage(backIm);

    randomFold();
  }  // end of useCard()

   
  private BufferedImage loadCardImage(int cardIdx, int face)
  {
    String numStr = (cardIdx < 10) ? "0" + cardIdx : "" + cardIdx;
    if (face == FRONT)
      return loadImage("images/f" + numStr + "_f.jpg");
    else   // back face
      return loadImage("images/f" + numStr + "_b.jpg");
  }  // end of loadCardImage()



  private void useRandomCards(int n)
  {
    // System.out.println("useRandomCards: " + n);
    int cardIdx =  rand.nextInt(names.length) + 1;

    frontIm = loadCardImage(cardIdx, FRONT);
    frontCard.setImage(frontIm);

    if (n == 2) // two random cards
      cardIdx =  rand.nextInt(names.length) + 1;

    backIm = loadCardImage(cardIdx, BACK);
    backCard.setImage(backIm);

    randomFold();
  }  // end of useRandomCards()



  private void randomFold()
  {
    BufferedImage topIm = null;
    BufferedImage botIm = null;

    while ((currTPos == topPos) && (currBPos == botPos)) {
      // get a random foldee Panel for the top
      topIm = getRandomPanel(TOP);
      
      // get a random foldee Panel for the bottom
      botIm = getRandomPanel(BOTTOM);
    }

    currTPos = topPos; currBPos = botPos;
    foldeePane.setImage( joinImages(topIm, botIm) );
  }  // end of randomFold()



  private BufferedImage getRandomPanel(int yPos)
  {
    // select an xPos for the given yPos of the foldee

    int xPos = rand.nextInt(3);   // 0, 1, or 2
    if (yPos == TOP)
      topPos = xPos;
    else  // BOTTOM
      botPos = xPos;
    
    // get the foldee Panel defined by (xPos, yPos)
    if ((xPos == LEFT) || (xPos == RIGHT))
      return getCardPanel(backIm, xPos, yPos);
    else  // CENTER
      return getCardPanel(frontIm, xPos, yPos);
  }  // end of getRandomPanel()



  private BufferedImage getCardPanel(BufferedImage im,
                              int xPos, int yPos)
  { if (xPos == LEFT) {
      if (yPos == TOP)
        return cropImage(im, 0, 0, 
                             CARD_WIDTH/3, CARD_HEIGHT/2);
      else  // BOTTOM
        return cropImage(im, 0, CARD_HEIGHT/2, 
                             CARD_WIDTH/3, CARD_HEIGHT/2);
    }
    else if (xPos == CENTER) {
      if (yPos == TOP)
        return cropImage(im, CARD_WIDTH/3, 0, 
                             CARD_WIDTH/3, CARD_HEIGHT/2);
      else  // BOTTOM
        return cropImage(im, CARD_WIDTH/3, CARD_HEIGHT/2, 
                             CARD_WIDTH/3, CARD_HEIGHT/2);
    }
    else {   // RIGHT panel
      if (yPos == TOP)
        return cropImage(im, CARD_WIDTH*2/3, 0, 
                             CARD_WIDTH/3, CARD_HEIGHT/2);
      else  // BOTTOM
        return cropImage(im, CARD_WIDTH*2/3, CARD_HEIGHT/2, 
                             CARD_WIDTH/3, CARD_HEIGHT/2);
    }
  }  // end of getCardPanel()



  // ------------------ image manipulation ------------

  private BufferedImage loadImage(String fnm)
  // load an image file and return it as a BufferedImage.
  {
    try {
      BufferedImage image = ImageIO.read(new File(fnm));
      if (image == null || (image.getWidth(null) < 0))
        // we could not load the image - probably invalid file format
        return null;
      return image;
    }
    catch (IOException e)
    {  System.out.println(e); }

    return null;
  }  // end of loadImage()



  private BufferedImage cropImage(BufferedImage im, 
                           int x, int y, int w, int h)
  { 
    int imWidth = im.getWidth(null);
    int imHeight = im.getHeight(null);

    // avoid cropping out of bounds
    if ((x+w) >= imWidth)
      w = imWidth-1 - x;

    if ((y+h) >= imHeight)
      h = imHeight-1 - y;
    
    if (im == null || (im.getWidth(null) < 0))
      return null;
    else
      return im.getSubimage(x, y, w, h);
  }  // end of cropImage()



  public static BufferedImage joinImages(
                      BufferedImage topIm, BufferedImage botIm) 
  {
    // create a buffer for new image
    BufferedImage im = new BufferedImage(CARD_WIDTH/3, CARD_HEIGHT,
                                      BufferedImage.TYPE_INT_ARGB);
    Graphics2D g2 = im.createGraphics();
    g2.setPaint(Color.WHITE);

    //draw image
    g2.drawImage(topIm, null, 0, 0);
    g2.drawImage(botIm, null, 0, CARD_HEIGHT/2);
    g2.dispose();
    return im;
  }  // end of joinImages()



  // -----------------------------------------

  public static void main(String[] args)
  { 
    try {
      UIManager.setLookAndFeel(
            "com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel");
    } 
    catch(Exception e) {}
    new Foldees();
  }   // end of main()


} // end of Foldees class


